@@ -11,6 +11,7 @@ __all__ = (
 
 import os
 import json
+import re
 from distutils.version import LooseVersion
 
 import kobo.pkgset
@@ -40,7 +41,7 @@ def to_utf8(text):
     encodings = ["ascii", "utf8", "latin1", "latin2"]
     for encoding in encodings:
         try:
-            return text.decode(encoding).encode("utf-8")
+            return text.decode(encoding)
         except UnicodeDecodeError:
             pass
     return text.decode("ascii", "ignore")
@@ -60,7 +61,7 @@ class SimpleRpmWrapperWithChangelog(kobo.pkgset.SimpleRpmWrapper):
         ts = kwargs.pop("ts", None)
         header = kobo.rpmlib.get_rpm_header(file_path, ts=ts)
 
-        self.summary = kobo.rpmlib.get_header_field(header, "summary")
+        self.summary = kobo.rpmlib.get_header_field(header, "summary").decode('utf-8')
 
         if self.sourcerpm:
             key = self.sourcerpm
@@ -104,10 +105,10 @@ def get_changelog_diff_from_headers(old, new, max_records=-1):
     if new_changelog and old_changelog:
         result = []
         old_time = old_changelog[0].time
-        old_nvr = LooseVersion(old_changelog[0].name.rsplit(None, 1)[-1])
+        old_nvr = LooseVersion(to_utf8(old_changelog[0].name).rsplit(None, 1)[-1])
         while new_changelog:
             entry = new_changelog.pop(0)
-            new_nvr = LooseVersion(entry.name.rsplit(None, 1)[-1])
+            new_nvr = LooseVersion(to_utf8(entry.name).rsplit(None, 1)[-1])
             if entry.time < old_time or (entry.time == old_time and new_nvr <= old_nvr):
                 # We want to take all entries from new changelog that are newer
                 # than old changelog, or from the same day as latest old entry
@@ -149,7 +150,11 @@ def get_srpm(lst, name, rpm_obj_list):
 
 
 class ComposeChangelog(object):
-    def __init__(self):
+    def __init__(self, strip_suffix=None):
+        self.suffix_re = None
+        if strip_suffix:
+            self.suffix_re = re.compile('\.%s[0-9a-fA-F]+$' % strip_suffix)
+
         self.file_cache = kobo.pkgset.FileCache(SimpleRpmWrapperWithChangelog)
 
     def get_srpms(self, compose):
@@ -158,7 +163,7 @@ class ComposeChangelog(object):
         for variant in rm:
             for arch in rm[variant]:
                 for srpm_nevra in rm[variant][arch]:
-                    for rpm_nevra, data in rm[variant][arch][srpm_nevra].iteritems():
+                    for rpm_nevra, data in rm[variant][arch][srpm_nevra].items():
                         if data['category'] != 'source':
                             continue
                         result.setdefault(
@@ -200,7 +205,7 @@ class ComposeChangelog(object):
         for variant in rm:
             for arch in rm[variant]:
                 for srpm_nevra in rm[variant][arch]:
-                    for rpm_nevra, data in rm[variant][arch][srpm_nevra].iteritems():
+                    for rpm_nevra, data in rm[variant][arch][srpm_nevra].items():
                         if data["category"] in ("debug", "source"):
                             continue
                         result.setdefault(
@@ -337,6 +342,12 @@ class ComposeChangelog(object):
             new_package = get_srpm(srpm_headers_new, name, rpm_headers_new[name])
             old_package = get_srpm(srpm_headers_old, name, rpm_headers_old[name])
 
+            if self.suffix_re:
+                # We need to compare releases without considering hash from
+                # module build.
+                if self.suffix_re.sub('', new_package.nvr) == self.suffix_re.sub('', old_package.nvr):
+                    continue
+
             data = {}
             data["name"] = new_package.name
             data["nvr"] = new_package.nvr
@@ -355,7 +366,7 @@ class ComposeChangelog(object):
 
             data["changelog"] = []
             for i in get_changelog_diff_from_headers(old_package, new_package, max_logs):
-                data["changelog"].append(to_utf8(str(i)))
+                data["changelog"].append("* %s %s\n%s" % (i.ctime, to_utf8(i.name), to_utf8(i.text)))
 
             # TODO: comps, system release
             # if rpm.versionCompare(old_package, new_package.header) == -1:
@@ -521,7 +532,7 @@ class ComposeChangelog(object):
             result.append("")
         result.append("")
 
-        return "\n".join([to_utf8(i) for i in result])
+        return "\n".join(result)
 
     def write(self, data, path=None, name=None, verbose=False):
         if name:
@@ -546,5 +557,6 @@ class ComposeChangelog(object):
 
         # verbose
         verbose_log = os.path.join(path, "%s.verbose" % name)
-        with open(verbose_log, "w") as f:
-            f.write(self.get_verbose_log(data, shorten=verbose))
+        with open(verbose_log, "wb") as f:
+            clog = self.get_verbose_log(data, shorten=verbose)
+            f.write(clog.encode('utf-8'))
